<?php
require_once __DIR__ . '/core.php';
require_once __DIR__ . '/filters.php';
require_once __DIR__ . '/menu.php';
require_once __DIR__ . '/pages.php';
require_once __DIR__ . '/hooks.php';

defined( 'ABSPATH' ) || exit;

final class SmartMailer {

	public $plugin_slug;
	public $version;

	public function __construct() {
		$this->plugin_slug = plugin_basename( SMARTMAILER_ROOT_DIR );
		$this->version     = '1.1.1';

		register_activation_hook( SM_PLUGIN_FILE, array( 'SmartMailer', 'activate' ) );
		add_filter( 'plugins_api', array( $this, 'pluginInfo' ), 20, 3 );
		add_filter( 'site_transient_update_plugins', array( $this, 'updatePlugin' ) );
		add_action( 'upgrader_process_complete', array( $this, 'purgeCache' ), 10, 2 );

		add_action( 'admin_init', array( $this, 'admin_init' ) );
	}

	public function admin_init(): void {
		if ( isset( $_GET['page'], $_GET['do_smartmailer_export'] ) && $_GET['page'] === 'smartmailer-settings' && $_GET['do_smartmailer_export'] === 'true' ) {
			SmartMailerFunctions::exportCsv();
		}
	}

	public static function activate() {
		// Send email to SM admins to notify about installation
		wp_mail(
			'info@smartmailer.cz',
			__( 'Smartmailer WP plugin nainstalován', 'smartmailer' ),
			__( 'SmartMailer WordPress plugin byl právě nainstalován na ' . get_site_url() . '.', 'smartmailer' )
		);
	}

	public function updateRequest() {
		$remote = wp_remote_get(
			'https://utils.smartmailer.cz/wp-plugin-updater/smartmailer_plugin_updater.php',
			array(
				'timeout' => 2,
				'headers' => array(
					'Accept' => 'application/json',
				),
			)
		);

		if (
			is_wp_error( $remote )
			|| 200 !== wp_remote_retrieve_response_code( $remote )
			|| empty( wp_remote_retrieve_body( $remote ) )
		) {
			return false;
		}

		$remote = json_decode( wp_remote_retrieve_body( $remote ) );

		return $remote;
	}

	function pluginInfo( $res, $action, $args ) {
		if ( 'plugin_information' !== $action ) {
			return $res;
		}

		if ( $this->plugin_slug !== $args->slug ) {
			return $res;
		}

		$remote = $this->updateRequest();

		if ( ! $remote ) {
			return $res;
		}

		$res = new stdClass();

		$res->name           = $remote->name;
		$res->slug           = $remote->slug;
		$res->version        = $remote->version;
		$res->tested         = $remote->tested;
		$res->requires       = $remote->requires;
		$res->author         = $remote->author;
		$res->author_profile = $remote->author_profile;
		$res->download_link  = $remote->download_url;
		$res->trunk          = $remote->download_url;
		$res->requires_php   = $remote->requires_php;
		$res->last_updated   = $remote->last_updated;

		$res->sections = array(
			'description'  => $remote->sections->description,
			'installation' => $remote->sections->installation,
			'changelog'    => $remote->sections->changelog,
		);

		if ( ! empty( $remote->banners ) ) {
			$res->banners = array(
				'low'  => $remote->banners->low,
				'high' => $remote->banners->high,
			);
		}

		return $res;
	}

	public function updatePlugin( $transient ) {

		if ( empty( $transient->checked ) ) {
			return $transient;
		}

		$remote = $this->updateRequest();

		if (
			$remote
			&& version_compare( $this->version, $remote->version, '<' )
			&& version_compare( $remote->requires, get_bloginfo( 'version' ), '<=' )
			&& version_compare( $remote->requires_php, PHP_VERSION, '<' )
		) {
			$res              = new stdClass();
			$res->slug        = $this->plugin_slug;
			$res->plugin      = plugin_basename( SMARTMAILER_ROOT_DIR . '/smartmailer.php' );
			$res->new_version = $remote->version;
			$res->tested      = $remote->tested;
			$res->package     = $remote->download_url;

			$transient->response[ $res->plugin ] = $res;

		}

		return $transient;
	}

	public function purgeCache( $upgrader, $options ) {

		if (
			'update' === $options['action']
			&& 'plugin' === $options['type']
		) {
			$files = glob( SMARTMAILER_CACHE_DIR );
			foreach ( $files as $file ) {
				if ( $file !== 'index.php' && is_file( $file ) ) {
					unlink( $file );
				}
			}
		}
	}
}
