<?php
declare( strict_types=1 );

require_once __DIR__ . '/Api.php';

class SmartMailerFunctions {

	public static function sendToSmartMailer( array $data ): void {
		$apiToken        = get_option( 'smartmailer_api_token', null );
		$addressBookUuid = get_option( 'smartmailer_subscribe_uuid', null );

		if ( ! empty( $apiToken ) && ! empty( $addressBookUuid ) ) {
			$api = new Api( $apiToken );
			$api->insertContacts( $addressBookUuid, self::getSerializedData( $data ) );
		}
	}

	private static function getSerializedData( $data ): array {
		$result = array();

		foreach ( $data as $key => $contact ) {
			$result[ $key ] = $contact['value'];
		}

		return array(
			'contacts' => array(
				$result,
			),
		);
	}

	public static function isWooCommerceActive(): bool {
		return is_plugin_active( 'woocommerce/woocommerce.php' );
	}

	public static function isContactForm7Active(): bool {
		return is_plugin_active( 'contact-form-7/wp-contact-form-7.php' );
	}

	public static function isWPFormsActive(): bool {
		return is_plugin_active( 'wpforms-lite/wpforms.php' ) || is_plugin_active( 'wpforms/wpforms.php' ) || is_plugin_active( 'wpforms-pro/wpforms.php' );
	}

	public static function getWPUserFields(): array {
		return array(
			'EMAIL' => array(
				'key'   => 'email',
				'value' => null,
			),
			'FNAME' => array(
				'key'   => 'first_name',
				'value' => null,
			),
			'LNAME' => array(
				'key'   => 'last_name',
				'value' => null,
			),
		);
	}

	public static function getWCFields(): array {
		return array(
			'EMAIL' => array(
				'key'   => 'email',
				'value' => null,
			),
			'FNAME' => array(
				'key'   => 'first_name',
				'value' => null,
			),
			'LNAME' => array(
				'key'   => 'last_name',
				'value' => null,
			),
		);
	}

	public static function getCF7Fields( int $form_id = 0 ): array {
		$form_options = get_option( 'sm_cf7_' . $form_id, array() );

		if ( empty( $form_options ) ) {
			return array(
				'EMAIL' => array(
					'key'   => 'email',
					'value' => null,
				),
				'FNAME' => array(
					'key'   => 'firstname',
					'value' => null,
				),
				'LNAME' => array(
					'key'   => 'lastname',
					'value' => null,
				),
			);
		}

		$result = array();

		foreach ( $form_options as $key => $option ) {
			if ( ! empty( $option ) ) {
				$result[ $option ] = array(
					'key'   => $key,
					'value' => null,
				);
			}
		}

		return $result;
	}

	public static function getWPFormsFields( int $form_id = 0 ): array {
		$form_options = get_option( 'sm_wpforms_' . $form_id, array() );

		if ( empty( $form_options ) ) {
			return array(
				'EMAIL' => array(
					'key'   => 'E-mail',
					'value' => null,
				),
				'FNAME' => array(
					'key'   => 'Jméno',
					'value' => null,
				),
				'LNAME' => array(
					'key'   => 'Příjmení',
					'value' => null,
				),
			);
		}

		$result = array();

		foreach ( $form_options as $key => $option ) {
			if ( ! empty( $option ) ) {
				$result[ $option ] = array(
					'key'   => $key,
					'value' => null,
				);
			}
		}

		return $result;
	}

	private static function getWoocommerceData(): array {
		$data = array();

		$orders = wc_get_orders(
			array(
				'limit' => - 1,
			)
		);

		if ( ! empty( $orders ) ) {
			foreach ( $orders as $order ) {
				if ( $order instanceof WC_Order ) {
					$data[] = array(
						$order->get_billing_email(),
						$order->get_billing_first_name(),
						$order->get_billing_last_name(),
					);
				}
			}
		}

		return $data;
	}

	private static function getUsersData(): array {
		$data = array();

		$users = get_users(
			array(
				'fields'       => array( 'display_name', 'user_email' ),
				'role__not_in' => array( 'administrator' ),
			)
		);

		if ( ! empty( $users ) ) {
			foreach ( $users as $user ) {
				$names     = explode( ' ', $user->display_name );
				$firstName = $names[0] ?? '';
				$lastName  = $names[1] ?? '';
				$data[]    = array(
					$user->user_email,
					$firstName,
					$lastName,
				);
			}
		}

		return $data;
	}

	private static function getCommentData(): array {
		$data = array();

		$comments = get_comments();

		if ( ! empty( $comments ) ) {
			foreach ( $comments as $comment ) {
				$name   = $comment->comment_author;
				$name   = explode( ' ', $name );
				$data[] = array(
					$comment->comment_author_email,
					$name[0] ?? '',
					$name[1] ?? '',
				);
			}
		}

		return $data;
	}

	private static function getFullData(): array {
		$data = array();

		if ( (bool) get_option( 'smartmailer_int_wordpress', false ) === true ) {
			$data = array_merge( $data, self::getUsersData() );
			$data = array_merge( $data, self::getCommentData() );
		}
		if ( self::isWooCommerceActive() && (bool) get_option( 'smartmailer_int_woocommerce', false ) === true ) {
			$data = array_merge( $data, self::getWoocommerceData() );
		}

		return $data;
	}

	public static function exportCsv(): void {
		$delimiter = ';';
		$header    = array(
			'EMAIL',
			'FNAME',
			'LNAME',
		);

		$file = fopen( 'php://output', 'wb+' );

		fputcsv( $file, $header, $delimiter );

		$data = self::getFullData();
		if ( ! empty( $data ) ) {
			foreach ( $data as $contact ) {
				fputcsv( $file, $contact, $delimiter );
			}
		}

		fclose( $file );

		header( 'Content-type: text/csv' );
		header( 'Content-Disposition: attachment; filename="smartmailer_' . ( new DateTime() )->format( 'Y-m-d' ) . '.csv"' );

		die();
	}

	public static function get_wpforms_forms(): array {
		$args = array(
			'post_type'      => 'wpforms',
			'posts_per_page' => - 1,
			'post_status'    => 'publish',
		);

		return ( new WP_Query( $args ) )->get_posts();
	}

	public static function get_cf7_forms(): array {
		$args = array(
			'post_type'      => 'wpcf7_contact_form',
			'posts_per_page' => - 1,
			'post_status'    => 'publish',
		);

		return ( new WP_Query( $args ) )->get_posts();
	}
}
